import validator from '@navikt/fnrvalidator'
import i18n from 'i18next'
import { isValidPhoneNumber } from 'libphonenumber-js'

const validEmailRegex =
  /^(?=.{1,254}$)(?=.{1,64}@)[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+(\.[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+)*@[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?(\.[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?)*$/

export function getCookie(name: string) {
  if (!document.cookie) {
    return null
  }

  const cookies = document.cookie
    .split(';')
    .map((c) => c.trim())
    .filter((c) => c.startsWith(`${name}=`))

  if (cookies.length === 0) {
    return null
  }
  return decodeURIComponent(cookies[0].split('=')[1])
}

export function maybeCsrfToken() {
  const csrfToken = getCookie('csrftoken')
  if (!csrfToken) {
    return null
  }
  return {
    'X-CSRFToken': csrfToken,
  }
}

export function submitJsonOpts(method: string, data: object): RequestInit {
  return {
    method,
    headers: {
      'Content-Type': 'application/json',
      ...maybeCsrfToken(),
    },
    body: JSON.stringify(data),
    credentials: 'same-origin',
  }
}

export function isValidFnr(data: string | undefined): boolean | string {
  if (data === undefined) {
    return i18n.t<string>('common:validation.invalidIdNumber').toString()
  }
  const valid = validator.idnr(data as string).status === 'valid'
  if (valid) {
    return true
  }
  // TypeScript complains if toString is not used on the function result
  return i18n.t<string>('common:validation.invalidIdNumber').toString()
}

export function isValidMobilePhoneNumber(
  data: string | undefined
): boolean | string {
  if (!data) {
    return i18n.t<string>('common:validation.invalidMobilePhoneNumber')
  }
  const valid = isValidPhoneNumber(data)
  if (valid) {
    return true
  }
  return i18n.t<string>('common:validation.invalidMobilePhoneNumber')
}

export function isValidEmail(data: string | undefined): boolean | string {
  if (!data) {
    return i18n.t<string>('common:validation.emailRequired')
  }
  if (validEmailRegex.test(data)) {
    return true
  }
  return i18n.t<string>('common:validation.invalidEmail')
}
